@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -122,11 +122,11 @@ cm_DirPrefetchBuffers(cm_dirOp_t * op);
 /* compute how many 32 byte entries an AFS 3 dir requires for storing
  * the specified name.
  */
-long 
+long
 cm_NameEntries(char *namep, size_t *lenp)
 {
     long i;
-        
+
     i = (long)strlen(namep);
     if (lenp) *lenp = i;
     return 1 + ((i+16) >> 5);
@@ -445,7 +445,7 @@ cm_DirFindBlobs(cm_dirOp_t * op, int nblobs)
     return -1;
 }
 
-/* Add a page to a directory. 
+/* Add a page to a directory.
 
    Called with op->scp->rw
 */
@@ -848,7 +848,7 @@ cm_DirGetBlob(cm_dirOp_t * op,
     *blobpp = (cm_dirEntry_t *) (ep + 32 * (blobno & (CM_DIR_EPP - 1)));
 
     return code;
-}	
+}
 
 int
 cm_DirHash(char *string)
@@ -972,18 +972,18 @@ cm_DirFindItem(cm_dirOp_t * op,
     }
 }
 
-/* Begin a sequence of directory operations.  
+/* Begin a sequence of directory operations.
  * Called with scp->rw unlocked.
  */
 long
 cm_BeginDirOp(cm_scache_t * scp, cm_user_t * userp, cm_req_t * reqp,
-              afs_uint32 lockType, cm_dirOp_t * op)
+              afs_uint32 lockType, afs_uint32 flags, cm_dirOp_t * op)
 {
     long code;
     int i, mxheld = 0, haveWrite = 0;
 
-    osi_Log3(afsd_logp, "Beginning dirOp[0x%p] for scp[0x%p], userp[0x%p]",
-             op, scp, userp);
+    osi_Log4(afsd_logp, "Beginning dirOp[0x%p] for scp[0x%p], userp[0x%p] lockType[0x%x]",
+             op, scp, userp, lockType);
 
     memset(op, 0, sizeof(*op));
 
@@ -1003,7 +1003,7 @@ cm_BeginDirOp(cm_scache_t * scp, cm_user_t * userp, cm_req_t * reqp,
     if (lockType == CM_DIRLOCK_WRITE) {
         lock_ObtainWrite(&scp->dirlock);
         haveWrite = 1;
-    } else { 
+    } else {
         lock_ObtainRead(&scp->dirlock);
         haveWrite = 0;
     }
@@ -1019,7 +1019,7 @@ cm_BeginDirOp(cm_scache_t * scp, cm_user_t * userp, cm_req_t * reqp,
 #ifdef USE_BPLUS
         if (!cm_BPlusTrees ||
             (scp->dirBplus &&
-             scp->dirDataVersion == scp->dataVersion)) 
+             scp->dirDataVersion == scp->dataVersion))
         {
             /* we know that haveWrite matches lockType at this point */
             switch (lockType) {
@@ -1036,9 +1036,10 @@ cm_BeginDirOp(cm_scache_t * scp, cm_user_t * userp, cm_req_t * reqp,
             default:
                 osi_assert(haveWrite);
             }
+            op->lockType = lockType;
         } else {
-            if (!(scp->dirBplus && 
-                  scp->dirDataVersion == scp->dataVersion)) 
+            if (!(scp->dirBplus &&
+                  scp->dirDataVersion == scp->dataVersion))
             {
               repeat:
                 if (!haveWrite) {
@@ -1053,7 +1054,7 @@ cm_BeginDirOp(cm_scache_t * scp, cm_user_t * userp, cm_req_t * reqp,
                     lock_ObtainWrite(&scp->rw);
                     mxheld = 1;
                 }
-                if (scp->dirBplus && 
+                if (scp->dirBplus &&
                      scp->dirDataVersion != scp->dataVersion)
                 {
                     bplus_dv_error++;
@@ -1063,44 +1064,55 @@ cm_BeginDirOp(cm_scache_t * scp, cm_user_t * userp, cm_req_t * reqp,
                     scp->dirDataVersion = CM_SCACHE_VERSION_BAD;
                 }
 
-                if (!scp->dirBplus) {
+                if ((!scp->dirBplus) &&
+                    (!(flags & CM_DIROP_FLAG_NOBUILDTREE))) {
                     if (mxheld) {
                         lock_ReleaseWrite(&scp->rw);
                         mxheld = 0;
                     }
-                    cm_BPlusDirBuildTree(scp, userp, reqp);
+                    code = cm_BPlusDirBuildTree(scp, userp, reqp);
+                    osi_Log1(afsd_logp, "cm_BeginDirOp cm_BPlusDirBuildTree code 0x%x", code);
                     if (!mxheld) {
                         lock_ObtainWrite(&scp->rw);
                         mxheld = 1;
                     }
-                    if (op->dataVersion != scp->dataVersion) {
-                        /* We lost the race, therefore we must update the
-                         * dirop state and retry to build the tree.
-                         */
-                        op->length = scp->length;
-                        op->newLength = op->length;
-                        op->dataVersion = scp->dataVersion;
-                        op->newDataVersion = op->dataVersion;
-                        goto repeat;
+                    if (code) {
+                        bplus_free_tree++;
+                        freeBtree(scp->dirBplus);
+                        scp->dirBplus = NULL;
+                        scp->dirDataVersion = CM_SCACHE_VERSION_BAD;
+                    } else {
+                        if (op->dataVersion != scp->dataVersion) {
+                            /* We lost the race, therefore we must update the
+                             * dirop state and retry to build the tree.
+                            */
+                            op->length = scp->length;
+                            op->newLength = op->length;
+                            op->dataVersion = scp->dataVersion;
+                            op->newDataVersion = op->dataVersion;
+                            goto repeat;
+                        }
+
+                         if (scp->dirBplus)
+                            scp->dirDataVersion = scp->dataVersion;
                     }
+                } /* build btree */
+            }
 
-                    if (scp->dirBplus)
-                        scp->dirDataVersion = scp->dataVersion;
+            if (code == 0) {
+                switch (lockType) {
+                case CM_DIRLOCK_NONE:
+                    lock_ReleaseWrite(&scp->dirlock);
+                    break;
+                case CM_DIRLOCK_READ:
+                    lock_ConvertWToR(&scp->dirlock);
+                    break;
+                case CM_DIRLOCK_WRITE:
+                default:
+                    /* got it already */;
                 }
+                op->lockType = lockType;
             }
-
-            switch (lockType) {
-            case CM_DIRLOCK_NONE:
-                lock_ReleaseWrite(&scp->dirlock);
-                break;
-            case CM_DIRLOCK_READ:
-                lock_ConvertWToR(&scp->dirlock);
-                break;
-            case CM_DIRLOCK_WRITE:
-            default:
-                /* got it already */;
-            }
-            haveWrite = 0;
         }
 #else
         /* we know that haveWrite matches lockType at this point */
@@ -1118,18 +1130,21 @@ cm_BeginDirOp(cm_scache_t * scp, cm_user_t * userp, cm_req_t * reqp,
         default:
             osi_assert(haveWrite);
         }
-#endif
         op->lockType = lockType;
-        if (mxheld)
-            lock_ReleaseWrite(&scp->rw);
-    } else {
+#endif
+    }
+
+    if (mxheld)
+        lock_ReleaseWrite(&scp->rw);
+
+    if (code) {
         if (haveWrite)
             lock_ReleaseWrite(&scp->dirlock);
         else
             lock_ReleaseRead(&scp->dirlock);
-        if (mxheld)
-            lock_ReleaseWrite(&scp->rw);
         cm_EndDirOp(op);
+
+        osi_Log1(afsd_logp, "cm_BeginDirOp return code 0x%x", code);
     }
 
     return code;
@@ -1150,17 +1165,24 @@ cm_CheckDirOpForSingleChange(cm_dirOp_t * op)
     code = cm_DirCheckStatus(op, 1);
 
     if (code == 0 &&
-        op->dataVersion == op->scp->dataVersion - 1) {
-        /* only one set of changes happened between cm_BeginDirOp()
-           and this function.  It is safe for us to perform local
-           changes. */
+        op->dataVersion == op->scp->dataVersion - 1)
+    {
+        /*
+         * only one set of changes happened between cm_BeginDirOp()
+         * and this function.  It is safe for us to perform local
+         * changes. */
         op->newDataVersion = op->scp->dataVersion;
         op->newLength = op->scp->serverLength;
 
         rc = 1;
+    } else {
+        /*
+         * The directory buffers are no longer up to date.
+         */
+        rc = 0;
     }
-    lock_ReleaseWrite(&op->scp->rw); 
-    
+    lock_ReleaseWrite(&op->scp->rw);
+
     if (rc)
         osi_Log0(afsd_logp, "cm_CheckDirOpForSingleChange succeeded");
     else
@@ -1170,29 +1192,30 @@ cm_CheckDirOpForSingleChange(cm_dirOp_t * op)
     return rc;
 }
 
-/* End a sequence of directory operations.  
+/* End a sequence of directory operations.
  * Called with op->scp->rw unlocked.*/
 long
 cm_EndDirOp(cm_dirOp_t * op)
 {
     long code = 0;
 
+    osi_Log4(afsd_logp, "Ending dirOp[0x%p] scp[0x%p] lockType[0x%x] with %d dirty buffer releases",
+             op, op->scp, op->lockType, op->dirtyBufCount);
+
     if (op->scp == NULL)
         return 0;
 
-    osi_Log2(afsd_logp, "Ending dirOp 0x%p with %d dirty buffer releases",
-             op, op->dirtyBufCount);
-
     if (op->dirtyBufCount > 0) {
 #ifdef USE_BPLUS
         /* update the data version on the B+ tree */
-        if (op->scp->dirBplus && 
+        if (op->scp->dirBplus &&
              op->scp->dirDataVersion == op->dataVersion) {
 
             switch (op->lockType) {
             case CM_DIRLOCK_READ:
-                lock_ReleaseRead(&op->scp->dirlock);
-                /* fall through ... */
+                lock_ConvertRToW(&op->scp->dirlock);
+                op->lockType = CM_DIRLOCK_WRITE;
+                break;
             case CM_DIRLOCK_NONE:
                 lock_ObtainWrite(&op->scp->dirlock);
                 op->lockType = CM_DIRLOCK_WRITE;
@@ -1234,6 +1257,9 @@ cm_EndDirOp(cm_dirOp_t * op)
 
     osi_assertx(op->nBuffers == 0, "Buffer leak after dirOp termination");
 
+    if (code)
+        osi_Log1(afsd_logp, "cm_EndDirOp return code 0x%x", code);
+
     return code;
 }
 
@@ -1288,7 +1314,7 @@ cm_DirOpAddBuffer(cm_dirOp_t * op, cm_buf_t * bufferp)
 
         if (code == 0 && bufferp->dataVersion != op->dataVersion) {
                 osi_Log2(afsd_logp,
-                         "cm_DirOpAddBuffer: buffer data version mismatch. buf dv = %d. needs %d", 
+                         "cm_DirOpAddBuffer: buffer data version mismatch. buf dv = %d. needs %d",
                          bufferp->dataVersion, op->dataVersion);
 
                 cm_SyncOpDone(op->scp, bufferp,
@@ -1395,7 +1421,7 @@ cm_DirOpDelBuffer(cm_dirOp_t * op, cm_buf_t * bufferp, int flags)
                          (op->lockType == CM_DIRLOCK_WRITE ? CM_SCACHESYNC_WRITE : CM_SCACHESYNC_READ));
 
 #ifdef DEBUG
-            osi_assert(bufferp->dataVersion == op->dataVersion);
+            osi_assert(bufferp->dataVersion == op->dataVersion || bufferp->dataVersion == CM_BUF_VERSION_BAD);
 #endif
 
             lock_ReleaseWrite(&op->scp->rw);
@@ -1547,7 +1573,6 @@ cm_DirPrefetchBuffers(cm_dirOp_t * op)
         offset = LargeIntegerAdd(offset, ConvertLongToLargeInteger(cm_data.buf_blockSize));
     }
 
- done:
     lock_ReleaseWrite(&op->scp->rw);
 
     osi_Log1(afsd_logp, "cm_DirPrefetchBuffers returning code 0x%x", code);
