@@ -562,8 +562,8 @@ CkSrv_MarkUpDown(struct afs_conn **conns, int nconns, afs_int32 *results)
 }
 
 void
-CkSrv_SetTime(struct rx_connection **rxconns, int nconns, int nservers,
-	      struct afs_conn **conns, struct srvAddr **addrs)
+CkSrv_SetTime(int nconns, struct rx_connection **rxconns,
+              struct afs_conn **conns)
 {
     struct afs_conn *tc;
     afs_int32 start, end = 0, delta;
@@ -573,11 +573,11 @@ CkSrv_SetTime(struct rx_connection **rxconns, int nconns, int nservers,
     afs_int32 i = 0;
     char tbuffer[CVBS];
 
-    conntimer = afs_osi_Alloc(nservers * sizeof (afs_int32));
+    conntimer = afs_osi_Alloc(nconns * sizeof (afs_int32));
     osi_Assert(conntimer != NULL);
-    results = afs_osi_Alloc(nservers * sizeof (afs_int32));
+    results = afs_osi_Alloc(nconns * sizeof (afs_int32));
     osi_Assert(results != NULL);
-    deltas = afs_osi_Alloc(nservers * sizeof (afs_int32));
+    deltas = afs_osi_Alloc(nconns * sizeof (afs_int32));
     osi_Assert(deltas != NULL);
 
     /* make sure we're starting from zero */
@@ -601,7 +601,7 @@ CkSrv_SetTime(struct rx_connection **rxconns, int nconns, int nservers,
 		deltas[multi_i] = end - tv.tv_sec;
 	} multi_End;
     } else {			/* find and query setTimeHost only */
-	for ( i = 0 ; i < nservers ; i++ ) {
+	for ( i = 0 ; i < nconns ; i++ ) {
 	    if ( conns[i] == NULL || conns[i]->srvr == NULL )
 		continue;
 	    if ( conns[i]->srvr->server == afs_setTimeHost ) {
@@ -688,25 +688,25 @@ CkSrv_SetTime(struct rx_connection **rxconns, int nconns, int nservers,
 	    }
 	}
     }
-    afs_osi_Free(conntimer, nservers * sizeof(afs_int32));
-    afs_osi_Free(deltas, nservers * sizeof(afs_int32));
-    afs_osi_Free(results, nservers * sizeof(afs_int32));
+    afs_osi_Free(conntimer, nconns * sizeof(afs_int32));
+    afs_osi_Free(deltas, nconns * sizeof(afs_int32));
+    afs_osi_Free(results, nconns * sizeof(afs_int32));
 }
 
 void
-CkSrv_GetCaps(struct rx_connection **rxconns, int nconns, int nservers,
-	      struct afs_conn **conns, struct srvAddr **addrs)
+CkSrv_GetCaps(int nconns, struct rx_connection **rxconns,
+	      struct afs_conn **conns)
 {
     Capabilities *caps;
     afs_int32 *results;
     afs_int32 i;
     struct server *ts;
 
-    caps = afs_osi_Alloc(nservers * sizeof (Capabilities));
+    caps = afs_osi_Alloc(nconns * sizeof (Capabilities));
     osi_Assert(caps != NULL);
-    memset(caps, 0, nservers * sizeof(Capabilities));
+    memset(caps, 0, nconns * sizeof(Capabilities));
 
-    results = afs_osi_Alloc(nservers * sizeof (afs_int32));
+    results = afs_osi_Alloc(nconns * sizeof (afs_int32));
     osi_Assert(results != NULL);
 
     AFS_GUNLOCK();
@@ -718,7 +718,7 @@ CkSrv_GetCaps(struct rx_connection **rxconns, int nconns, int nservers,
     AFS_GLOCK();
 
     for ( i = 0 ; i < nconns ; i++ ) {
-	ts = addrs[i]->server;
+	ts = conns[i]->srvr->server;
 	if ( !ts )
 	    continue;
 	ts->capabilities = 0;
@@ -739,8 +739,8 @@ CkSrv_GetCaps(struct rx_connection **rxconns, int nconns, int nservers,
     }
     CkSrv_MarkUpDown(conns, nconns, results);
 
-    afs_osi_Free(caps, nservers * sizeof(Capabilities));
-    afs_osi_Free(results, nservers * sizeof(afs_int32));
+    afs_osi_Free(caps, nconns * sizeof(Capabilities));
+    afs_osi_Free(results, nconns * sizeof(afs_int32));
 }
 
 /* check down servers (if adown), or running servers (if !adown) */
@@ -756,14 +756,12 @@ afs_CheckServers(int adown, struct cell *acellp)
  *        AFS_LS_ALL  - check all */
 void
 afs_LoopServers(int adown, struct cell *acellp, int vlalso,
-		void (*func1) (struct rx_connection **rxconns, int nconns,
-			       int nservers, struct afs_conn **conns,
-			       struct srvAddr **addrs),
-		void (*func2) (struct rx_connection **rxconns, int nconns,
-			       int nservers, struct afs_conn **conns,
-			       struct srvAddr **addrs))
+		void (*func1) (int nservers, struct rx_connection **rxconns,
+		               struct afs_conn **conns),
+		void (*func2) (int nservers, struct rx_connection **rxconns,
+			       struct afs_conn **conns))
 {
-    struct vrequest treq;
+    struct vrequest *treq = NULL;
     struct server *ts;
     struct srvAddr *sa;
     struct afs_conn *tc = NULL;
@@ -775,7 +773,7 @@ afs_LoopServers(int adown, struct cell *acellp, int vlalso,
     struct afs_conn **conns;
     int nconns;
     struct rx_connection **rxconns;
-    afs_int32 *conntimer, *results;
+    afs_int32 *conntimer;
 
     AFS_STATCNT(afs_CheckServers);
 
@@ -791,7 +789,7 @@ afs_LoopServers(int adown, struct cell *acellp, int vlalso,
     conntimer = 0;
     nconns = 0;
 
-    if ((code = afs_InitReq(&treq, afs_osi_credp)))
+    if ((code = afs_CreateReq(&treq, afs_osi_credp)))
 	return;
     ObtainReadLock(&afs_xserver);	/* Necessary? */
     ObtainReadLock(&afs_xsrvAddr);
@@ -823,8 +821,6 @@ afs_LoopServers(int adown, struct cell *acellp, int vlalso,
     osi_Assert(rxconns != NULL);
     conntimer = afs_osi_Alloc(j * sizeof (afs_int32));
     osi_Assert(conntimer != NULL);
-    results = afs_osi_Alloc(j * sizeof (afs_int32));
-    osi_Assert(results != NULL);
 
     for (i = 0; i < j; i++) {
 	struct rx_connection *rxconn;
@@ -847,7 +843,7 @@ afs_LoopServers(int adown, struct cell *acellp, int vlalso,
 	/* check vlserver with special code */
 	if (sa->sa_portal == AFS_VLPORT) {
 	    if (vlalso)
-		CheckVLServer(sa, &treq);
+		CheckVLServer(sa, treq);
 	    continue;
 	}
 
@@ -855,7 +851,7 @@ afs_LoopServers(int adown, struct cell *acellp, int vlalso,
 	    continue;		/* have just been added by setsprefs */
 
 	/* get a connection, even if host is down; bumps conn ref count */
-	tu = afs_GetUser(treq.uid, ts->cell->cellNum, SHARED_LOCK);
+	tu = afs_GetUser(treq->uid, ts->cell->cellNum, SHARED_LOCK);
 	tc = afs_ConnBySA(sa, ts->cell->fsport, ts->cell->cellNum, tu,
 			  1 /*force */ , 1 /*create */ , SHARED_LOCK, 0,
 			  &rxconn);
@@ -878,10 +874,13 @@ afs_LoopServers(int adown, struct cell *acellp, int vlalso,
 	    afs_PutConn(tc, rxconn, SHARED_LOCK);
     } /* Outer loop over addrs */
 
-    (*func1)(rxconns, nconns, j, conns, addrs);
+    afs_osi_Free(addrs, srvAddrCount * sizeof(*addrs));
+    addrs = NULL;
+
+    (*func1)(nconns, rxconns, conns);
 
     if (func2) {
-	(*func2)(rxconns, nconns, j, conns, addrs);
+	(*func2)(nconns, rxconns, conns);
     }
 
     for (i = 0; i < nconns; i++) {
@@ -890,11 +889,10 @@ afs_LoopServers(int adown, struct cell *acellp, int vlalso,
 	afs_PutConn(conns[i], rxconns[i], SHARED_LOCK);     /* done with it now */
     }
 
-    afs_osi_Free(addrs, srvAddrCount * sizeof(*addrs));
     afs_osi_Free(conns, j * sizeof(struct afs_conn *));
     afs_osi_Free(rxconns, j * sizeof(struct rx_connection *));
     afs_osi_Free(conntimer, j * sizeof(afs_int32));
-    afs_osi_Free(results, j * sizeof(afs_int32));
+    afs_DestroyReq(treq);
 
 } /*afs_CheckServers*/
 
@@ -1784,7 +1782,7 @@ void
 afs_GetCapabilities(struct server *ts)
 {
     Capabilities caps = {0, NULL};
-    struct vrequest treq;
+    struct vrequest *treq = NULL;
     struct afs_conn *tc;
     struct unixuser *tu;
     struct rx_connection *rxconn;
@@ -1795,16 +1793,20 @@ afs_GetCapabilities(struct server *ts)
     if ( !afs_osi_credp )
 	return;
 
-    if ((code = afs_InitReq(&treq, afs_osi_credp)))
+    if ((code = afs_CreateReq(&treq, afs_osi_credp)))
 	return;
-    tu = afs_GetUser(treq.uid, ts->cell->cellNum, SHARED_LOCK);
-    if ( !tu )
+    tu = afs_GetUser(treq->uid, ts->cell->cellNum, SHARED_LOCK);
+    if ( !tu ) {
+	afs_DestroyReq(treq);
 	return;
+    }
     tc = afs_ConnBySA(ts->addr, ts->cell->fsport, ts->cell->cellNum, tu, 0, 1,
 		      SHARED_LOCK, 0, &rxconn);
     afs_PutUser(tu, SHARED_LOCK);
-    if ( !tc )
+    if ( !tc ) {
+	afs_DestroyReq(treq);
 	return;
+    }
     /* InitCallBackStateN, triggered by our RPC, may need this */
     ReleaseWriteLock(&afs_xserver);
     code = RXAFS_GetCapabilities(rxconn, &caps);
@@ -1818,6 +1820,7 @@ afs_GetCapabilities(struct server *ts)
     if ( code && code != RXGEN_OPCODE ) {
 	afs_warn("RXAFS_GetCapabilities failed with code %d\n", code);
 	/* better not be anything to free. we failed! */
+	afs_DestroyReq(treq);
 	return;
     }
 
@@ -1830,6 +1833,7 @@ afs_GetCapabilities(struct server *ts)
 	caps.Capabilities_val = NULL;
     }
 
+    afs_DestroyReq(treq);
 }
 
 static struct server *
