@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -138,11 +138,11 @@ IDHash(afs_int32 x)
 }
 
 static afs_int32
-NameHash(register char *aname)
+NameHash(char *aname)
 {
     /* returns hash bucket for aname */
-    register unsigned int hash = 0;
-    register int i;
+    unsigned int hash = 0;
+    int i;
 /* stolen directly from the HashString function in the vol package */
     for (i = strlen(aname), aname += i - 1; i--; aname--)
 	hash = (hash * 31) + (*(unsigned char *)aname - 31);
@@ -217,7 +217,7 @@ readUbikHeader(struct misc_data *misc)
     }
 
     uheader.magic = ntohl(uheader.magic);
-    uheader.size = ntohl(uheader.size);
+    uheader.size = ntohs(uheader.size);
     uheader.version.epoch = ntohl(uheader.version.epoch);
     uheader.version.counter = ntohl(uheader.version.counter);
 
@@ -267,6 +267,13 @@ PrintEntryError(struct misc_data *misc, afs_int32 ea, struct prentry *e, int ind
     return 0;
 }
 
+int
+PrintContError(struct misc_data *misc, afs_int32 ea, struct contentry *c, int indent)
+{
+    pr_PrintContEntry(stderr, /*net order */ 0, ea, c, indent);
+    return 0;
+}
+
 afs_int32
 WalkHashTable(afs_int32 hashtable[],	/* hash table to walk */
 	      int hashType,		/* hash function to use */
@@ -392,7 +399,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
     afs_int32 head;
     int bit;
     afs_int32 code;
-    struct prentry c;		/* continuation entry */
+    struct contentry c;		/* continuation entry */
     afs_int32 na;		/* next thread */
     int ni;
     afs_int32 eid = 0;
@@ -500,7 +507,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 		return PRDBBAD;
 	    if (na != sghead) {
 		fprintf(stderr, "last block: \n");
-		if (PrintEntryError(misc, na, &c, 4))
+		if (PrintContError(misc, na, &c, 4))
 		    return PRDBBAD;
 	    }
 	    return 0;
@@ -514,7 +521,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 	    fprintf(stderr, "Continuation entry reused\n");
 	    if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintEntryError(misc, na, &c, 4))
+	    if (PrintContError(misc, na, &c, 4))
 		return PRDBBAD;
 	    noErrors = 0;
 	    break;
@@ -524,7 +531,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 	    fprintf(stderr, "Continuation id mismatch\n");
 	    if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintEntryError(misc, na, &c, 4))
+	    if (PrintContError(misc, na, &c, 4))
 		return PRDBBAD;
 	    noErrors = 0;
 	    continue;
@@ -536,23 +543,19 @@ WalkNextChain(char map[],		/* one byte per db entry */
 	    if (id == PRBADID)
 		continue;
 	    else if (id) {
-		int eid_s, id_s;
+		int id_s;
 		sgcount++;
 		/* in case the ids are large, convert to pure sign. */
 		if (id > 0)
 		    id_s = 1;
 		else
 		    id_s = -1;
-		if (eid > 0)
-		    eid_s = 1;
-		else
-		    eid_s = -1;
 		if (id_s > 0) {
 		    fprintf(stderr,
 			    "User can't be member of supergroup list\n");
 		    if (PrintEntryError(misc, ea, e, 2))
 			return PRDBBAD;
-		    if (PrintEntryError(misc, na, &c, 4))
+		    if (PrintContError(misc, na, &c, 4))
 			return PRDBBAD;
 		    noErrors = 0;
 		}
@@ -579,7 +582,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 		return PRDBBAD;
 	    if (na != head) {
 		fprintf(stderr, "last block: \n");
-		if (PrintEntryError(misc, na, &c, 4))
+		if (PrintContError(misc, na, &c, 4))
 		    return PRDBBAD;
 	    }
 	    return 0;
@@ -595,7 +598,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 		fprintf(stderr, "walking free list");
 	    else if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintEntryError(misc, na, &c, 4))
+	    if (PrintContError(misc, na, &c, 4))
 		return PRDBBAD;
 	    noErrors = 0;
 	    break;
@@ -607,7 +610,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 		fprintf(stderr, "walking free list");
 	    else if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintEntryError(misc, na, &c, 4))
+	    if (PrintContError(misc, na, &c, 4))
 		return PRDBBAD;
 	    noErrors = 0;
 	    continue;
@@ -637,7 +640,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 				"User can't be member of user in membership list\n");
 			if (PrintEntryError(misc, ea, e, 2))
 			    return PRDBBAD;
-			if (PrintEntryError(misc, na, &c, 4))
+			if (PrintContError(misc, na, &c, 4))
 			    return PRDBBAD;
 			noErrors = 0;
 		    }
@@ -647,7 +650,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 				"Bad user/group dicotomy in membership list\n");
 			if (PrintEntryError(misc, ea, e, 2))
 			    return PRDBBAD;
-			if (PrintEntryError(misc, na, &c, 4))
+			if (PrintContError(misc, na, &c, 4))
 			    return PRDBBAD;
 			noErrors = 0;
 		    }
@@ -709,7 +712,7 @@ WalkOwnedChain(char map[],		/* one byte per db entry */
 {
     afs_int32 head;
     afs_int32 code;
-    struct prentry c;		/* continuation entry */
+    struct prentry te;		/* next entry in owner chain */
     afs_int32 na;		/* next thread */
     int ni;
     afs_int32 eid = 0;
@@ -722,7 +725,7 @@ WalkOwnedChain(char map[],		/* one byte per db entry */
 	head = ntohl(cheader.orphan);
 
     length = 0;
-    for (na = head; na; na = ntohl(c.nextOwned)) {
+    for (na = head; na; na = ntohl(te.nextOwned)) {
 	code = ConvertDiskAddress(na, &ni);
 	if (code) {
 	    fprintf(stderr, "Bad owned list ptr %d", na);
@@ -732,23 +735,33 @@ WalkOwnedChain(char map[],		/* one byte per db entry */
 		return PRDBBAD;
 	    if (na != head) {
 		fprintf(stderr, "last block: \n");
-		if (PrintEntryError(misc, na, &c, 4))
+		if (PrintEntryError(misc, na, &te, 4))
 		    return PRDBBAD;
 	    }
 	    return 0;
 	}
-	code = pr_Read(na, (char *)&c, sizeof(c));
+	code = pr_Read(na, (char *)&te, sizeof(te));
 	if (code)
 	    return code;
 	length++;
 
+	if ((ntohl(te.flags) & PRTYPE) == PRCONT) {
+	    fprintf(stderr, "Continuation entry found on owner chain\n");
+	    if (e == 0)
+		fprintf(stderr, "walking orphan list");
+	    else if (PrintEntryError(misc, ea, e, 2))
+		return PRDBBAD;
+	    if (PrintEntryError(misc, na, &te, 4))
+		return PRDBBAD;
+	    break;
+	}
 	if (map[ni] & MAP_OWNED) {
 	    fprintf(stderr, "Entry on multiple owner chains\n");
 	    if (e == 0)
 		fprintf(stderr, "walking orphan list");
 	    else if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintEntryError(misc, na, &c, 4))
+	    if (PrintEntryError(misc, na, &te, 4))
 		return PRDBBAD;
 	    break;
 	}
@@ -760,16 +773,16 @@ WalkOwnedChain(char map[],		/* one byte per db entry */
 		fprintf(stderr, "walking orphan list");
 	    else if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintEntryError(misc, na, &c, 4))
+	    if (PrintEntryError(misc, na, &te, 4))
 		return PRDBBAD;
 	    continue;
 	}
 	if (e) {
-	    if (ntohl(c.owner) != eid) {
+	    if (ntohl(te.owner) != eid) {
 		fprintf(stderr, "Owner id mismatch\n");
 		goto abort;
 	    }
-	} else /* orphan */ if (c.owner) {
+	} else /* orphan */ if (te.owner) {
 	    fprintf(stderr, "Orphan group owner not zero\n");
 	    goto abort;
 	}
@@ -931,12 +944,12 @@ GC(char map[], struct misc_data *misc)
 	    id = ntohl(e.id);
 #if defined(SUPERGROUPS)
 	    if ((id != ANONYMOUSID)
-		&& ((refCount = idcount(&misc->idmap, id)) != ntohl(e.count))) 
+		&& ((refCount = idcount(&misc->idmap, id)) != ntohl(e.count)))
 #else
 	    if ((id >= misc->minId) && (id <= misc->maxId)
 		&& (id != ANONYMOUSID)
 		&& ((refCount = misc->idmap[id - misc->minId]) !=
-		    ntohl(e.count))) 
+		    ntohl(e.count)))
 #endif /* SUPERGROUPS */
 	      {
 		afs_int32 na;
@@ -1040,9 +1053,9 @@ DumpRecreate(char map[], struct misc_data *misc)
 		/* check for duplicate id.  This may still lead to duplicate
 		 * names. */
 #if defined(SUPERGROUPS)
-		if (idcount(&idmap, id)) 
+		if (idcount(&idmap, id))
 #else
-		if (idmap[id - misc->minId]) 
+		if (idmap[id - misc->minId])
 #endif
 		  {
 		    fprintf(stderr, "Skipping entry with duplicate id %di\n",
@@ -1151,9 +1164,9 @@ DumpRecreate(char map[], struct misc_data *misc)
 
 	    owner = ntohl(e.owner);
 #if defined(SUPERGROUPS)
-	    if (!idcount(&idmap, owner)) 
+	    if (!idcount(&idmap, owner))
 #else
-	    if (idmap[owner - misc->minId] == 0) 
+	    if (idmap[owner - misc->minId] == 0)
 #endif
 	      {
 		fprintf(stderr,
@@ -1180,9 +1193,6 @@ DumpRecreate(char map[], struct misc_data *misc)
 	    if ((id < 0) && (flags & PRGRP)) {
 		int count = 0;
 		afs_int32 na;
-#if defined(SUPERGROUPS)
-		afs_int32 ng;
-#endif
 		int i;
 		for (i = 0; i < PRSIZE; i++) {
 		    afs_int32 uid = ntohl(e.entries[i]);
@@ -1201,46 +1211,9 @@ DumpRecreate(char map[], struct misc_data *misc)
 				id);
 #endif
 		}
-#if defined(SUPERGROUPS)
-#define g	(*((struct prentryg *)&e))
-		ng = ntohl(g.nextsg);
-		for (i = 0; i < SGSIZE; i++) {
-		    afs_int32 uid = ntohl(g.supergroup[i]);
-		    if (uid == 0)
-			break;
-		    if (uid == PRBADID)
-			continue;
-		    fprintf(rc, "au %d %d\n", uid, id);
-		    count++;
-		}
-		while (ng) {
-		    struct prentry c;
-		    code = pr_Read(ng, (char *)&c, sizeof(c));
-		    if (code)
-			return code;
-
-		    if ((id == ntohl(c.id)) && (c.flags & htonl(PRCONT))) {
-			for (i = 0; i < COSIZE; i++) {
-			    afs_int32 uid = ntohl(c.entries[i]);
-			    if (uid == 0)
-				break;
-			    if (uid == PRBADID)
-				continue;
-			    fprintf(rc, "au %d %d\n", uid, id);
-			    count++;
-			}
-		    } else {
-			fprintf(stderr, "Skipping continuation block at %d\n",
-				ng);
-			break;
-		    }
-		    ng = ntohl(c.next);
-		}
-#undef g
-#endif /* SUPERGROUPS */
 		na = ntohl(e.next);
 		while (na) {
-		    struct prentry c;
+		    struct contentry c;
 		    code = pr_Read(na, (char *)&c, sizeof(c));
 		    if (code)
 			return code;
@@ -1423,7 +1396,7 @@ CheckPrDatabase(struct misc_data *misc)	/* info & statistics */
     free(map);
     return code;
 }
-    
+
 #include "AFS_component_version_number.c"
 
 int
@@ -1530,12 +1503,11 @@ inccount(struct idused **idmapp, int id)
 	idmapp = &idmap->idnext;
     }
     if (!idmap) {
-	idmap = (struct idused *)malloc(sizeof *idmap);
+	idmap = calloc(1, sizeof *idmap);
 	if (!idmap) {
 	    perror("idmap");
 	    exit(1);
 	}
-	memset(idmap, 0, sizeof idmap);
 	idmap->idstart = id & ~(IDCOUNT - 1);
 	idmap->idnext = *idmapp;
 	*idmapp = idmap;
